from pydantic import BaseModel, StrictStr
from .dimension_value_attribute import DimensionValueAttribute
from .group_definition import GroupDefinition
from .result import Result


class CostExplorerResponse(BaseModel):
    NextPageToken: StrictStr
    GroupDefinitions: list[GroupDefinition] | None = None
    ResultsByTime: list[Result]
    DimensionValueAttributes: list[DimensionValueAttribute]


from pydantic import BaseModel, StrictStr, StrictBool
from .dimension_value_attribute import DimensionValueAttribute
from .result_group import ResultGroup
from .time_period import TimePeriod
from .total import Total

class Result(BaseModel):
    DimensionValueAttributes: list[DimensionValueAttribute]
    TimePeriod: TimePeriod
    Total: dict[StrictStr, Total]
    Groups: list[ResultGroup]
    Estimated: StrictBool



import datetime
from pydantic import StrictStr
from typing import Optional
from werkflow_aws.models.base import AWSBoto3Base
from werkflow_aws.models.parsing import (
    convert_key_to_boto3_arg, 
    convert_key_to_boto3_arg_upper_matching,
    key_contains_patterns
)
from typing import Optional, Literal, Dict


class AWSs3CreateMultipartUploadOptions(AWSBoto3Base):
    acl=Optional[
        Literal[
            'private',
            'public-read',
            'public-read-write',
            'authenticated-read',
            'aws-exec-read',
            'bucket-owner-read',
            'bucket-owner-full-control'
        ]
    ]=None
    cache_control: Optional[StrictStr]=None
    content_disposition: Optional[StrictStr]=None
    content_encoding: Optional[StrictStr]=None
    content_language: Optional[StrictStr]=None
    content_type: Optional[StrictStr]=None
    expires: Optional[datetime.datetime]=None
    grant_full_control: Optional[StrictStr]=None
    grant_read: Optional[StrictStr]=None
    grant_read_acp: Optional[StrictStr]=None
    grant_write_acp: Optional[StrictStr]=None
    metadata: Optional[Dict[StrictStr, StrictStr]]=None
    server_side_encryption: Optional[
        Literal[
            'AES256',
            'aws:kms',
            'aws:kms:dsse'
        ]
    ]=None
    storage_class: Optional[
        Literal[
            'STANDARD',
            'REDUCED_REDUNDANCY',
            'STANDARD_IA',
            'ONEZONE_IA',
            'INTELLIGENT_TIERING',
            'GLACIER',
            'DEEP_ARCHIVE',
            'OUTPOSTS',
            'GLACIER_IR',
            'SNOW',
            'EXPRESS_ONEZONE'
        ]
    ]=None
    website_redirect_location: Optional[StrictStr]=None
    sse_customer_algorithm: Optional[StrictStr]=None
    sse_customer_key: Optional[StrictStr]=None
    sse_kms_key_id: Optional[StrictStr]=None
    sse_kms_encryption_context: Optional[StrictStr]=None
    bucket_key_enabled: Optional[bool]=None
    tagging: Optional[StrictStr]=None
    object_lock_mode: Optional[
        Literal[
            'GOVERNANCE',
            'COMPLIANCE'
        ]
    ]=None
    object_lock_retain_until_date: Optional[datetime.datetime]=None
    object_lock_legal_hold_status: Optional[
        Literal[
            'ON',
            'OFF'
        ]
    ]=None
    expected_bucket_owner: Optional[StrictStr]=None
    checksum_algorithm: Optional[
        Literal[
            'CRC32',
            'CRC32C',
            'SHA1',
            'SHA256'
        ]
    ]=None

    def to_options(self):
        options = self._filtered_options_to_dict()

        uppercased_options = {
            convert_key_to_boto3_arg_upper_matching(
                key,
                [
                    'acl',
                    'md5', 
                    'crc32',
                    'crc32c',
                    'sha1',
                    'sha256',
                    'acp',
                    'sse',
                    'kms'
                ]
            ): value  for key, value in options.items() if key_contains_patterns(
                key,
                [
                    'acl',
                    'content_md5',
                    'checksum_crc32',
                    'checksum_crc32c',
                    'checksum_sha1',
                    'checksum_sha256',
                    'grant_read_acp',
                    'grant_write_acp',
                    'sse_customer_algorithm',
                    'sse_customer_key',
                    'sse_kms_key_id',
                    'sse_kms_encryption_context'
                ]
            )
        }

        parsed_options = {
            convert_key_to_boto3_arg(
                key
            ): value for key, value in options.items() if key not in uppercased_options
        }

        parsed_options.update(uppercased_options)

        return parsed_options





from pydantic import (
    StrictStr,
    StrictInt
)

from typing import Optional
from werkflow_aws.models.base import AWSBoto3Base


class AWSs3ListMultipartUploadsOptions(AWSBoto3Base):
    delimiter: Optional[StrictStr]=None
    encoding_type: Optional[StrictStr]=None
    key_marker: Optional[StrictStr]=None
    max_uploads: Optional[StrictInt]=None
    prefix: Optional[StrictStr]=None
    upload_id_marker: Optional[StrictStr]=None
    expected_bucket_owner: Optional[StrictStr]=None



import re
from pydantic import StrictFloat, BeforeValidator
from typing import Annotated

numeric_pattern = re.compile(r'(\d+.\d+)|(\d+)')

def prepare_amount(value: str | float) -> float:
    if isinstance(value, str) and (
        amount := re.match(
            numeric_pattern,
            value,
        )
    ):
        return float(amount.group(0))

    return value


Amount = Annotated[
    StrictFloat,
    BeforeValidator(prepare_amount),
]


from enum import Enum


class AWSRegion(Enum):
    AF_SOUTH_1='af-south-1'
    AP_EAST_1='ap-east-1'
    AP_NORTHEAST_1='ap-northeast-1'
    AP_NORTHEAST_2='ap-northeast-2'
    AP_NORTHEAST_3='ap-northeast-3'
    AP_SOUTH_1='ap-south-1'
    AP_SOUTH_2='ap-south-2'
    AP_SOUTHEAST_1='ap-southeast-1'
    AP_SOUTHEAST_2='ap-southeast-2'
    AP_SOUTHEAST_3='ap-southeast-3'
    AP_SOUTHEAST_4='ap-southeast-4'
    CA_CENTRAL_1='ca-central-1'
    EU_CENTRAL_1='eu-central-1'
    EU_CENTRAL_2='eu-central-2'
    EU_NORTH_1='eu-north-1'
    EU_SOUTH_1='eu-south-1'
    EU_SOUTH_2='eu-south-2'
    EU_WEST_1='eu-west-1'
    EU_WEST_2='eu-west-2'
    EU_WEST_3='eu-west-3'
    IL_CENTRAL_1='il-central-1'
    ME_CENTRAL_1='me-central-1'
    ME_SOUTH_1='me-south-1'
    SA_EAST_1='sa-east-1'
    US_EAST_1='us-east-1'
    US_EAST_2='us-east-2'
    US_GOV_EAST_1='us-gov-east-1'
    US_GOV_WEST_1='us-gov-west-1'
    US_WEST_1='us-west-1'
    US_WEST_2='us-west-2'



import datetime
from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool,
    StrictInt
)
from typing import Literal, Optional, Dict
from .aws_s3_streaming_body import AWSs3StreamingBody


class AWSs3GetObjectResponse(BaseModel):
    Body: AWSs3StreamingBody
    DeleteMarker: StrictBool
    AcceptRanges: StrictStr
    Expiration: StrictStr
    Restore: StrictStr
    LastModified: datetime.datetime
    ContentLength: StrictInt
    ETag: StrictStr
    ChecksumCRC32: StrictStr
    ChecksumCRC32C: StrictStr
    ChecksumSHA1: StrictStr
    ChecksumSHA256: StrictStr
    MissingMeta: StrictInt
    VersionId: StrictStr
    CacheControl: StrictStr
    ContentDisposition: StrictStr
    ContentEncoding: StrictStr
    ContentLanguage: StrictStr
    ContentRange: StrictStr
    ContentType: StrictStr
    Expires: datetime.datetime
    WebsiteREdirectLocation: StrictStr
    ServerSideEncryption: Literal[
        'AES256',
        'aws:kms',
        'aws:kms:dsse'
    ]
    Metadata: Dict[
        StrictStr,
        StrictStr
    ]
    SSECustomerAlgorithm: StrictStr
    SSECustomerKeyMD5: StrictStr
    SSECustomerId: StrictStr
    BucketKeyEnabled: StrictBool
    StorageClass: Literal[
        'STANDARD',
        'REDUCED_REDUNDANCY',
        'STANDARD_IA',
        'ONEZONE_IA',
        'INTELLIGENT_TIERING',
        'GLACIER',
        'DEEP_ARCHIVE',
        'OUTPOSTS',
        'GLACIER_IR',
        'SNOW',
        'EXPRESS_ONEZONE'
    ]
    ReqeustCharged: Optional[
        Literal['requester']
    ]=None
    ReplicationStatus: Literal[
        'COMPLETE',
        'PENDING',
        'FAILED',
        'REPLICA',
        'COMPLETED'
    ]=None
    PartsCount: StrictInt
    TagCount: StrictInt
    ObjectLockMode: Literal[
        'GOVERNANCE',
        'COMPLIANCE'
    ]
    ObjectLockRetainUntilDate: datetime.datetime
    ObjectLockLegalHoldStatus: Literal[
        'ON',
        'OFF'
    ]

    class Config:
        arbitrary_types_allowed=True


from pydantic import BaseModel, StrictStr


class DimensionValueAttribute(BaseModel):
    Value: StrictStr
    Attributes: dict[StrictStr, StrictStr]

    def dump(self):
        return self.model_dump()


import asyncio
import boto3
import json
import functools
from werkflow_aws.models import AWSCredentialsSet
from werkflow_secrets import Secrets
from werkflow.modules.shell import Shell
from typing import (
    Union, 
    Optional, 
    Dict, 
    Literal
)


class AWSCredentials:

    def __init__(self):

        self._error: Union[Exception, None] = None

        self._secrets = Secrets()
        self._shell = Shell()

        self.profile: Union[str, None] = None
        self.role: Union[str, None] = None
        self.account_id: Union[str, None] = None
        self.cache_filename: Union[str, None] = None
        self.access_token: Union[str, None] = None

        self.credentials: Union[AWSCredentialsSet, None] = None
    
    async def sso(
        self,
        profile_name: str,
    ):

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        await self._loop.run_in_executor(
            None,
            functools.partial(
                boto3.setup_default_session,
                profile_name=profile_name
            )
        )

    async def get_credentials(
        self,
        profile: Optional[str]=None,
        cache_path: Optional[str]=None
    ) -> AWSCredentialsSet:
        if profile is None:
            profile = await self._shell.get_envar('AWS_PROFILE', 'dev')

        if not self.cache_filename:
            self.cache_filename = await self.get_cache_filename(
                cache_path=cache_path
            )

        role = await self.get_role(
            profile=profile
        )

        account_id = await self.get_account_id(
            profile=profile
        )

        access_token = await self.get_access_token(
            cache_path=cache_path
        )

        command_parameters = ' '.join([
            f'--account-id={account_id}',
            f'--access-token={access_token}',
            '--output json',
            f'--profile={profile}',
            f'--role-name={role}'
        ])


        aws_role_credentials: Dict[str, str] = await self._shell.run(
            f'aws --profile={profile} sso get-role-credentials {command_parameters}'
        )

        if isinstance(aws_role_credentials, dict) is False:
            aws_role_credentials = {}

        aws_access_key_id = aws_role_credentials.get('accessKeyId')

        aws_secret_access_key = aws_role_credentials.get('secretAccessKey')

        aws_session_token = aws_role_credentials.get('sessionToken')
        
        self.credentials = AWSCredentialsSet(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token
        )

        return self.credentials

    async def get_credentials_from_env(self) -> AWSCredentialsSet:

        aws_access_key_id = await self._shell.get_envar('AWS_ACCESS_KEY_ID')
        aws_secret_access_key = await self._shell.get_envar('AWS_SECRET_ACCESS_KEY')
        aws_session_token = await self._shell.get_envar('AWS_SESSION_TOKEN')

        self.credentials = AWSCredentialsSet(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token
        )

        return self.credentials

    async def get_encrypted_credentials_from_env(
        self,
        encryption_method: Literal['basic', 'aes256gcm']='basic'
    ):

        await self._secrets.load_key_from_env()

        aws_access_key_id = await self._secrets.load_from_env(
            'AWS_ACCESS_KEY_ID',
            method=encryption_method
        )

        aws_secret_access_key = await self._secrets.load_from_env(
            'AWS_SECRET_ACCESS_KEY',
            method=encryption_method
        )

        aws_session_token = await self._secrets.load_from_env(
            'AWS_SESSION_TOKEN',
            method=encryption_method
        )

        return AWSCredentialsSet(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token
        )
    
    async def get_encrypted_credentials_from_file(
        self,
        access_key_id_path: Optional[str]=None,
        secret_access_key_path: Optional[str]=None,
        session_token_path: Optional[str]=None,
        keyname: str='secret',
        key_filepath: Optional[str]=None,
        encryption_method: Literal['basic', 'aes256gcm']='basic'
    ):

        await self._secrets.load_key_from_file(
            keyname=keyname,
            path=key_filepath
        )

        aws_access_key_id = await self._secrets.load_from_file(
            path=access_key_id_path,
            method=encryption_method
        )

        aws_secret_access_key = await self._secrets.load_from_file(
            path=secret_access_key_path,
            method=encryption_method
        )

        aws_session_token = await self._secrets.load_from_file(
            path=session_token_path,
            method=encryption_method
        )

        return AWSCredentialsSet(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token
        )

    async def get_role(
        self,
        profile: Optional[str]=None
    ):
        
        if profile is None:
            profile = await self._shell.get_envar('AWS_PROFILE', 'dev')

        process_result = await self._shell.run(
            f'aws --profile={profile} configure get sso_role_name configure get sso_account_id',
            silent=True
        )

        self.role = process_result.stdout

        return self.role
    
    async def get_account_id(
        self,
        profile: Optional[str]=None
    ):
        
        if profile is None:
            profile = await self._shell.get_envar('AWS_PROFILE', 'dev')

        process_result = await self._shell.run(
            f'aws --profile={profile} configure get sso_account_id',
            silent=True
        )

        self.account_id = process_result.stdout

        return self.account_id
    
    async def get_cache_filename(
        self,
        cache_path: Optional[str]=None
    ):
        
        if cache_path is None:
            cache_path = '~/.aws/sso/cache'
        
        cache_path = await self._shell.to_absolute_path(cache_path)

        process_result = await self._shell.run(
            f'grep -rl "accessToken" {cache_path}',
            silent=True
        )

        self.cache_filename = process_result

        return self.cache_filename
    
    async def get_access_token(
        self,
        cache_path: Optional[str]=None
    ):
        
        if not self.cache_filename:
            self.cache_filename = await self.get_cache_filename(
                cache_path=cache_path
            )

        cache_data = await self._shell.read_file(
            self.cache_filename,
            silent=True
        )

        data: Dict[str, str] = json.loads(cache_data)

        self.access_token = data.get('accessToken')

        if self.access_token is None:
            raise Exception(f'Err. - Access token not found in cache file {self.cache_filename}')
        
        return self.access_token

    async def close(self):
        await self._secrets.close()
        await self._shell.close()

    def abort(self):
        self._secrets.abort()
        self._shell.abort()


import datetime
from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt
)


class AWSs3PartResponse(BaseModel):
    PartNumber: StrictInt
    LastModified: datetime.datetime
    ETag: StrictStr
    Size: StrictInt
    ChecksumCRC32: StrictStr
    ChecksumCRC32C: StrictStr
    ChecksumSHA1: StrictStr
    ChecksumSHA256: StrictStr


from pydantic import (
    StrictStr
)

from typing import Optional
from werkflow_aws.models.base import AWSBoto3Base


class AWSs3ListDirectoryBucketsOptions(AWSBoto3Base):
    continuation_token: Optional[StrictStr]=None
    max_directory_buckets: Optional[StrictStr]=None



from abc import ABC, abstractmethod
from typing import Any, Literal

PolicyArn = dict[
    Literal["arn"],
    str,
]

Tag = dict[
    Literal['Key', 'Value'],
    str,
]

ProvidedContext = dict[
    Literal["ProviderArn", "ContextAssertion"],
    str,
]


    # RoleArn='string',
    # RoleSessionName='string',
    # PolicyArns=[
    #     {
    #         'arn': 'string'
    #     },
    # ],
    # Policy='string',
    # DurationSeconds=123,
    # Tags=[
    #     {
    #         'Key': 'string',
    #         'Value': 'string'
    #     },
    # ],
    # TransitiveTagKeys=[
    #     'string',
    # ],
    # ExternalId='string',
    # SerialNumber='string',
    # TokenCode='string',
    # SourceIdentity='string',
    # ProvidedContexts=[
    #     {
    #         'ProviderArn': 'string',
    #         'ContextAssertion': 'string'
    #     },
    # ]

class STSClient(ABC):

    @abstractmethod
    def assume_role(
        self,
        RoleArn: str,
        RoleSessionName: str,
        PolicyArns: list[PolicyArn] | None = None,
        Policy: str | None = None,
        DurationSeconds: int | None = None,
        Tags: list[Tag] | None = None,
        TransitiveTagKeys: list[str] | None = None,
        ExternalId: str | None = None,
        SerialNumber: str | None = None,
        TokenCode: str | None = None,
        SourceIdentity: str | None = None,
        ProvidedContexts: list[ProvidedContext] | None = None,
    ) -> dict[str, Any]:
        pass


from typing import List

def convert_key_to_boto3_arg(key: str):
    return ''.join([
        key_partial.capitalize() for key_partial in key.split('_')
    ])


def convert_key_to_boto3_arg_upper_matching(
    key: str,
    patterns: List[str]
):
    key_partials = key.split('_')

    partials: List[str] = []

    for partial in key_partials:
        if partial in patterns:
            partial = partial.upper()

        else:
            partial = partial.capitalize()

        partials.append(partial)

    return ''.join(partials)


def key_contains_patterns(
    key: str,
    patterns: List[str]
):
    return len([
        pattern for pattern in patterns if pattern in key
    ]) > 0


from .aws_codeartifact import AWSCodeArtifact as AWSCodeArtifact
from .aws_credentials import AWSCredentials as AWSCredentials
from .aws_cost_explorer import AWSCostExplorer as AWSCostExplorer
from .aws_elasticache import AWSElastiCache as AWSElastiCache
from .aws_sts import AWSSTS as AWSSTS


from __future__ import annotations
from abc import ABC, abstractmethod

class ElastiCacheClient(ABC):

    @abstractmethod
    def describe_cache_clusters(
        self,
        CacheClusterId: str = None,
        MaxRecords: int = None,
        Marker: str = None,
        ShowCacheNodeInfo: bool = True,
        ShowCacheClustersNotInReplicationGroups: bool = True,
    ) -> dict[str, str]:
        pass
        
    @abstractmethod
    def close(self):
        pass


from pydantic import (
    BaseModel,
    StrictStr
)
from typing import List, Optional
from .aws_s3_bucket_response import AWSs3BucketResponse


class AWSs3ListDirectoryBucketsResponse(BaseModel):
    Buckets: List[AWSs3BucketResponse]=[]
    ContinuationToken: Optional[StrictStr]=None


import datetime
from pydantic import BaseModel


class TimePeriod(BaseModel):
    Start: datetime.datetime | datetime.date
    End: datetime.datetime | datetime.date
    
    def dump(self, time_format: str | None = None):
        if time_format is None:
            time_format = '%Y-%m-%d'


        start = self.Start.strftime(time_format)
        end = self.End.strftime(time_format)

        return {
            'Start': start,
            'End': end
        }


from pydantic import (
    StrictStr,
    StrictInt
)
from werkflow_aws.models.base import AWSBoto3Base
from typing import Dict, Any, Optional


class AWSs3GeneratePresignedURLOptions(AWSBoto3Base):
    params: Optional[Dict[StrictStr, Any]]=None
    expires_in: Optional[StrictInt]=None
    http_method: Optional[StrictStr]=None


from .amount import Amount as Amount
from .cost_category import CostCategory as CostCategory
from .cost_explorer_query import CostExplorerQuery as CostExplorerQuery
from .cost_explorer_response import CostExplorerResponse as CostExplorerResponse
from .dimension_value_attribute import DimensionValueAttribute as DimensionValueAttribute
from .dimension import Dimension as Dimension
from .dump_options import DumpOptions as DumpOptions
from .expression import Expression as Expression
from .group_definition import GroupDefinition as GroupDefinition
from .group import Group as Group
from .metric import Metric as Metric
from .result_group import ResultGroup as ResultGroup
from .result import Result as Result
from .tag_value import TagValue as TagValue
from .time_period import TimePeriod as TimePeriod
from .total import Total as Total


import datetime
from pydantic import BaseModel, StrictStr


class Credentials(BaseModel):
    AccessKeyId: StrictStr
    SecretAccessKey: StrictStr
    SessionToken: StrictStr
    Expiration: datetime.datetime


from .aws_s3_abort_multipart_upload_options import AWSs3AbortMultipartUploadOptions
from .aws_s3_abort_multipart_upload_response import AWSs3AbortMultipartUploadResponse
from .aws_s3_complete_multipart_upload_options import AWSs3CompleteMultipartUploadOptions
from .aws_s3_copy_source import AWSs3CopySource
from .aws_s3_copy_object_options import AWSs3CopyObjectOptions
from .aws_s3_copy_object_response import AWSs3CopyObjectResponse
from .aws_s3_complete_multipart_upload_response import AWSs3CompleteMultipartUploadResponse
from .aws_s3_create_multipart_upload_options import AWSs3CreateMultipartUploadOptions
from .aws_s3_create_multipart_upload_response import AWSs3CreateMultipartUploadResponse
from .aws_s3_delete_bucket_options import AWSs3DeleteBucketOptions
from .aws_s3_delete_bucket_response import AWSs3DeleteBucketReponse
from .aws_s3_delete_bucket_tagging_options import AWSs3DeleteBucketTaggingOptions
from .aws_s3_delete_bucket_tagging_response import AWSs3DeleteBucketTaggingResponse
from .aws_s3_delete_object_options import AWSs3DeleteObjectOptions
from .aws_s3_delete_object_response import AWSs3DeleteObjectResponse
from .aws_s3_delete_object_tagging_options import AWSs3DeleteObjectTaggingOptions
from .aws_s3_delete_object_tagging_response import AWSs3DeleteObjectTaggingResponse
from .aws_s3_delete_objects_options import AWSs3DeleteObjectsOptions
from .aws_s3_delete_objects_response import AWSs3DeleteObjectsResponse
from .aws_s3_generate_presigned_post_options import AWSs3GeneratePresignedPostOptions
from .aws_s3_generate_presigned_post_response import AWSs3GeneratePresignedPostResponse
from .aws_s3_generate_presigned_url_options import AWSs3GeneratePresignedURLOptions
from .aws_s3_generate_presigned_url_response import AWSs3GeneratePresignedURLResponse
from .aws_s3_get_bucket_tagging_options import AWSs3GetBucketTaggingOptions
from .aws_s3_get_bucket_tagging_response import AWSs3GetBucketTaggingResponse
from .aws_s3_get_object_options import AWSs3GetObjectOptions
from .aws_s3_get_object_response import AWSs3GetObjectResponse
from .aws_s3_get_object_tagging_options import AWSs3GetObjectTaggingOptions
from .aws_s3_get_object_tagging_response import AWSs3GetObjectTaggingResponse
from .aws_s3_list_buckets import AWSs3ListBucketsResponse
from .aws_s3_list_directory_buckets_options import AWSs3ListDirectoryBucketsOptions
from .aws_s3_list_directory_buckets_response import AWSs3ListDirectoryBucketsResponse
from .aws_s3_list_objects_options import AWSs3ListObjectsOptions
from .aws_s3_list_objects_response import AWSs3ListObjectsResponse
from .aws_s3_list_object_versions_options import AWSs3ListObjectVersionsOptions
from .aws_s3_list_object_versions_response import AWSs3ListObjectVersionsResponse
from .aws_s3_list_parts_options import AWSs3ListPartsOptions
from .aws_s3_list_parts_response import AWSs3ListPartsResponse
from .aws_s3_list_multipart_uploads_options import AWSs3ListMultipartUploadsOptions
from .aws_s3_list_multipart_uploads_response import AWSs3ListMultipartUploadsResponse
from .aws_s3_multipart_upload import AWSs3MultipartUpload
from .aws_s3_object import AWSs3Object
from .aws_s3_put_bucket_tagging_options import AWSs3PutBucketTaggingOptions
from .aws_s3_put_bucket_tagging_response import AWSs3PutBucketTaggingResponse
from .aws_s3_put_bucket_versioning_options import AWSs3PutBucketVersioningOptions
from .aws_s3_put_bucket_versioning_response import AWSs3PutBucketVersioningResponse
from .aws_s3_put_object_options import AWSs3PutObjectOptions
from .aws_s3_put_object_response import AWSs3PutObjectResponse
from .aws_s3_put_object_tagging_options import AWSs3PutObjectTaggingOptions
from .aws_s3_put_object_tagging_response import AWSs3PutObjectTaggingResponse
from .aws_s3_streaming_body import AWSs3StreamingBody
from .aws_s3_tag import AWSs3Tag
from .aws_s3_transfer_allowed_upload_args import AWSs3TransferAllowedUploadArgs
from .aws_s3_transfer_config_options import AWSs3TransferConfigOptions
from .aws_s3_upload_part_copy_options import AWSs3UploadPartCopyOptions
from .aws_s3_upload_part_copy_response import AWSs3UploadPartCopyResponse
from .aws_s3_upload_part_options import AWSs3UploadPartOptions
from .aws_s3_upload_part_response import AWSs3UploadPartResponse


from pydantic import BaseModel
from werkflow_aws.models.parsing import convert_key_to_boto3_arg



class AWSBoto3Base(BaseModel):

    def _filtered_options_to_dict(self):

        options = self.model_dump()

        return {
            key: value for key, value in options.items() if value is not None
        }

    def to_data(self):
        return {
            convert_key_to_boto3_arg(
                key
            ): value for key, value in self._filtered_options_to_dict()
        }




from pydantic import (
    BaseModel,
    StrictStr,
    AnyHttpUrl
)
from typing import Dict


class AWSs3GeneratePresignedPostResponse(BaseModel):
    url: AnyHttpUrl
    fields: Dict[StrictStr, StrictStr]


import datetime
from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool
)
from werkflow_aws.models.base import AWSBoto3Base
from typing import Dict, Literal


class AWSs3DeleteMarker(AWSBoto3Base):
    Owner: Dict[
        Literal[
            'DisplayName',
            'ID'
        ],
        StrictStr
    ]
    Key: StrictStr
    VersionId: StrictStr
    IsLatest: StrictBool
    LastModified: datetime.datetime


import datetime
from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
    StrictBool
)
from werkflow_aws.models.base import AWSBoto3Base
from typing import Literal, Dict, List


class AWSs3ObjectVersion(AWSBoto3Base):
    e_tag: StrictStr
    checksum_algorithm: List[
        Literal[
            'CRC32',
            'CRC32C',
            'SHA1',
            'SHA256'
        ]
    ]=[]
    size: StrictInt
    storage_class: Literal[
        'STANDARD'
    ]
    key: StrictStr
    version_id: StrictStr
    is_latest: StrictBool
    last_modified: datetime.datetime
    owner: Dict[
        Literal[
            'DisplayName',
            'ID'
        ],
        StrictStr
    ]
    restore_status: Dict[
        Literal[
            'IsRestoreInProgress',
            'RestoreExpiryDate'
        ],
        StrictStr
    ]


from .assume_role_request import AssumeRoleRequest as AssumeRoleRequest
from .assume_role_response import AssumedRoleResponse as AssumedRoleResponse
from .assumed_role_user import AssumedRoledUser as AssumedRoledUser
from .credentials import Credentials as Credentials
from .policy_arn import PolicyArn as PolicyArn
from .provided_context import ProvidedContext as ProvidedContext
from .tag import Tag as Tag


import asyncio
import boto3
import functools
from botocore.config import Config
from concurrent.futures import ThreadPoolExecutor
from werkflow_aws.exceptions import (
    EmptyResponseException,
    UnsetAWSConnectionException
)
from werkflow_aws.models import (
    AWSCredentialsSet,
    AWSRegion
)
from werkflow_aws.types import (
    CodeArtifactClient,
    CodeArtifactFormat
)
from werkflow.modules.system import System
from typing import Union


class AWSCodeArtifact:
    
    def __init__(self) -> None:
        
        self._system = System()

        self._loop: Union[asyncio.AbstractEventLoop, None] = None
        self._executor = ThreadPoolExecutor(
            max_workers=self._system.configuration.cores.physical
        )

        self._client = None

        self.service_name = 'CodeArtifact'

    async def sso(
        self,
        profile_name: str,
    ):

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.setup_default_session,
                profile_name=profile_name
            )
        )

        self._client: CodeArtifactClient = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.client,
                'codeartifact',
            )
        )


    async def connect(
        self,
        credentials: AWSCredentialsSet,
        region: AWSRegion
    ):

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        self._client: CodeArtifactClient = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.client,
                'codeartifact',
                aws_access_key_id=credentials.aws_access_key_id,
                aws_secret_access_key=credentials.aws_secret_access_key,
                aws_session_token=credentials.aws_session_token,
                config=Config(
                    region=region.value
                )
            )
        )

    async def get_authorization_token(
        self,
        domain: str,
        domain_owner: str,
        duration_seconds: int=1000
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )

        result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_authorization_token,
                domain=domain,
                domainOwner=domain_owner,
                durationSeconds=duration_seconds
            )
        )

        authorization_token = result.get('authorizationToken')

        if authorization_token is None:
            raise EmptyResponseException(
                self.service_name,
                'get_authorization_token',
                'authorizationToken'
            )
        
        return authorization_token
    
    async def get_repository_endpoint(
        self,
        domain: str,
        domain_owner: str,
        repository: str,
        repository_format: CodeArtifactFormat
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            ) 
        
        result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_repository_endpoint,
                domain=domain,
                domainOwner=domain_owner,
                repository=repository,
                format=repository_format
            )
        )

        endpoint = result.get('repositoryEndpoint')

        if endpoint is None:
            raise EmptyResponseException(
                self.service_name,
                'get_repository_endpoint',
                'repositoryEndpoint'
            )

        return endpoint

    async def close(self):

        await self._loop.run_in_executor(
            None,
            self._client.close
        )

        await self._system.close()
        self._executor.shutdown()

    def abort(self):
        self._client.close()
        self._system.abort()
        self._executor.shutdown()


from pydantic import (
    BaseModel,
    StrictStr
)
from typing import Optional


class AWSCredentialsSet(BaseModel):
    aws_access_key_id: StrictStr
    aws_secret_access_key: StrictStr
    aws_session_token: StrictStr


from __future__ import annotations
from abc import ABC, abstractmethod
from typing import Literal, Any

MatchOption = Literal[
    "EQUALS",
    "ABSENT",
    "STARTS_WITH",
    "ENDS_WITH",
    "CONTAINS",
    "CASE_SENSITIVE",
    "CASE_INSENSITIVE",
    "GREATER_THAN_OR_EQUAL",
]

Matchable = dict[
    Literal[
        'Key',
        'MatchOptions',
        'Values'
    ],
    str | MatchOption | list[str]
]


ExpressionFields = Literal[
    'And',
    'CostCategories',
    'Dimensions',
    'Or',
    'Not',
    'Tags',
]

ExpressionValues = dict[ExpressionFields, Any] | Matchable

Expression = dict[ExpressionFields, ExpressionValues]


GroupMethod = Literal[
    'DIMENSION',
    'TAG',
    'COST_CATEGORY',
]

Group = dict[GroupMethod, str]


class CostExplorerClient(ABC):

    @abstractmethod
    def get_cost_and_usage(
        self,
        TimePeriod: dict[str, str] = None,
        Granularity: Literal['DAILY', 'MONTHLY', 'HOURLY'] = 'MONTHLY',
        Metrics: list[str] = None,
        Filter: Expression | None = None,
        GroupBy: list[Group] | None = None,
        BillingViewArn: str | None = None,
        NextPageToken: str | None = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def close(self):
        pass


import datetime
from pydantic import (
    StrictStr,
    StrictInt
)

from typing import Optional, Literal, List
from werkflow_aws.models.base import AWSBoto3Base


class AWSs3ListObjectVersionsOptions(AWSBoto3Base):
    delimiter: Optional[StrictStr]=None,
    encodying_type: Optional[Literal['url']]=None,
    key_marker: Optional[StrictStr]=None,
    max_keys: Optional[StrictInt]=None,
    prefix: Optional[StrictStr]=None,
    version_id_marker: Optional[StrictStr]=None,
    expected_bucket_owner: Optional[StrictStr]=None,
    request_payer: Optional[Literal['requester']]=None,
    optional_object_attributes: Optional[List[StrictStr]]=None




from pydantic import BaseModel, StrictStr
from .amount import Amount


class Metric(BaseModel):
    Amount: Amount
    Unit: StrictStr

    def model_dump(self, *, mode = 'python', include = None, exclude = None, context = None, by_alias = None, exclude_unset = False, exclude_defaults = False, exclude_none = False, round_trip = False, warnings = True, fallback = None, serialize_as_any = False):
        dumped = super().model_dump(mode=mode, include=include, exclude=exclude, context=context, by_alias=by_alias, exclude_unset=exclude_unset, exclude_defaults=exclude_defaults, exclude_none=exclude_none, round_trip=round_trip, warnings=warnings, fallback=fallback, serialize_as_any=serialize_as_any)
        dumped['Amount'] = str(dumped['Amount'])
        
        return dumped


from pydantic import (
    BaseModel,
    StrictStr
)
from typing import Dict, Any


class AWSs3DeleteObjectTaggingResponse(BaseModel):
    VersionId: StrictStr
    ResponseMetadata: Dict[StrictStr, Any]


from pydantic import BaseModel, StrictStr, Field
from .match_option_types import MatchOptionTypes


class TagValue(BaseModel):
    Key: StrictStr = Field(pattern=r'[\S\s]*')
    MatchOptions: list[MatchOptionTypes]
    Values: list[StrictStr] = Field(pattern=r'[\S\s]*')

    def dump(self):
        return self.model_dump()


from pydantic import BaseModel, StrictStr, StrictInt
from .credentials import Credentials
from .assumed_role_user import AssumedRoledUser


class AssumedRoleResponse(BaseModel):
    Credentials: Credentials
    AssumedRoledUser: AssumedRoledUser
    PackedPolicySize: StrictInt
    SourceIdentity: StrictStr



import asyncio
import boto3
import functools
from botocore.config import Config
from concurrent.futures import ThreadPoolExecutor
from werkflow_aws.exceptions import UnsetAWSConnectionException
from werkflow_aws.models import (
    AWSCredentialsSet,
    AWSRegion,
)
from werkflow_aws.models.cost_explorer import CostExplorerQuery, CostExplorerResponse
from werkflow_aws.types import (
    CostExplorerClient,
    STSClient,
)
from werkflow.modules.system import System
from typing import Union


class AWSCostExplorer:
    
    def __init__(self) -> None:
        
        self._system = System()

        self._loop: Union[asyncio.AbstractEventLoop, None] = None
        self._executor = ThreadPoolExecutor(
            max_workers=self._system.configuration.cores.physical
        )

        self._client = None
        self._session: boto3.Session | None = None

        self.service_name = 'CostExplorer'

    async def connect(
        self,
        credentials: AWSCredentialsSet,
        region: AWSRegion
    ):

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        self._client: CostExplorerClient = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.client,
                'ce',
                aws_access_key_id=credentials.aws_access_key_id,
                aws_secret_access_key=credentials.aws_secret_access_key,
                aws_session_token=credentials.aws_session_token,
                config=Config(
                    region=region.value
                )
            )
        )

    async def sso(
        self,
        profile_name: str,
    ):

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.setup_default_session,
                profile_name=profile_name
            )
        )

        self._client: CostExplorerClient = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.client,
                'ce',
            )
        )

    async def assume_role(
        self,
        role_arn: str,
        role_session_name: str,
        role_external_id: str,
        region_name: str,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        self._session = boto3.Session(region_name=region_name)
        sts_client: STSClient = self._session.client('sts', region_name=region_name)
        
        response = sts_client.assume_role(
            RoleArn=role_arn,
            RoleSessionName=role_session_name,
            ExternalId=role_external_id
        )
        self.current_role_details = {
            'role_arn': role_arn,
            'external_id': role_external_id,
            'source_profile': 'N/A'  # Assuming source_profile is not available here
        }

    async def get_cost_and_usage(
        self,
        query: CostExplorerQuery
    ) -> CostExplorerResponse:
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name,
            )
        
        dumped = query.dump()
        
        result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_cost_and_usage,
                **dumped
            )
        )

        return CostExplorerResponse(**result)
    
    async def close(self):

        await self._loop.run_in_executor(
            None,
            self._client.close
        )

        await self._system.close()
        self._executor.shutdown()

    def abort(self):
        self._client.close()
        self._system.abort()
        self._executor.shutdown()


from pydantic import (
    StrictStr,
    StrictInt,
    constr
)
from typing import Dict, List, Any, Optional, Literal
from werkflow_aws.models.base import AWSBoto3Base


class AWSs3GeneratePresignedPostOptions(AWSBoto3Base):
    fields: Optional[
        Dict[str, Any]
    ]=None
    conditions: List[
        Dict[
            Literal[
                'acl',
                'content-length-range',
                'Cache-Control',
                'Content-Type',
                'Content-Disposition',
                'Content-Encoding',
                'Expires',
                'success_action_redirect',
                'redirect',
                'success_action_status',
            ] | constr(pattern=r'^(x-amz-meta-)(.*)'),
            StrictStr | StrictInt
        ] | List[
            Literal[
                'acl',
                'content-length-range',
                'Cache-Control',
                'Content-Type',
                'Content-Disposition',
                'Content-Encoding',
                'Expires',
                'success_action_redirect',
                'redirect',
                'success_action_status',
            ] | constr(pattern=r'^(x-amz-meta-)(.*)') |
            StrictStr | StrictInt
        ]
    ]


import datetime
from pydantic import StrictStr
from typing import Optional
from werkflow_aws.models.base import AWSBoto3Base
from werkflow_aws.models.parsing import (
    convert_key_to_boto3_arg, 
    convert_key_to_boto3_arg_upper_matching,
    key_contains_patterns
)
from typing import Optional, Literal, Dict


class AWSs3UploadPartCopyOptions(AWSBoto3Base):
    copy_source:  str | Dict[
        Literal['bucket','key','version_id'],
        str
    ]=None,
    copy_source_if_match: Optional[StrictStr]=None
    copy_source_if_modified_since: Optional[datetime.datetime]=None
    copy_source_if_none_match: Optional[StrictStr]=None
    copy_source_if_unmodified_since: Optional[datetime.datetime]=None
    copy_source_range: Optional[StrictStr]=None
    sse_customer_algorithm: Optional[StrictStr]=None
    sse_customer_key: Optional[StrictStr]=None
    copy_source_sse_customer_algorithm: Optional[StrictStr]=None
    copy_source_sse_customer_key: Optional[StrictStr]=None
    request_payer: Optional[
        Literal['requester']
    ]=None
    expected_bucket_owner: Optional[StrictStr]=None
    expected_source_bucket_owner: Optional[StrictStr]=None

    def to_options(self):
        options = self._filtered_options_to_dict()

        copy_source = options.get('copy_source')
        if isinstance(copy_source, dict):
            copy_source = {
                convert_key_to_boto3_arg(
                    key
                ): value for key, value in copy_source.items()
            }

            options['copy_source'] = copy_source

        uppercased_options = {
            convert_key_to_boto3_arg_upper_matching(
                key,
                [
                    'sse'
                ]
            ): value  for key, value in options.items() if key_contains_patterns(
                key,
                [
                    'content_md5',
                    'checksum_crc32',
                    'checksum_crc32c',
                    'checksum_sha1',
                    'checksum_sha256',
                    'sse_customer_algorithm',
                    'sse_customer_key',
                    'sse_kms_encryption_context',
                    'copy_source_sse_customer_algorithm',
                    'copy_source_sse_customer_key'
                ]
            )
        }

        parsed_options = {
            convert_key_to_boto3_arg(
                key
            ): value for key, value in options.items() if key not in uppercased_options
        }

        parsed_options.update(uppercased_options)

        return parsed_options





from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool
)
from typing import Literal, Optional
from .aws_s3_copy_object_result import AWSs3CopyOBjectResult


class AWSs3CopyObjectResponse(BaseModel):
    CopyObjectResult: AWSs3CopyOBjectResult
    Expiration: StrictStr
    CopySourceVersionId: StrictStr
    VersionId: StrictStr
    ServerSideEncryption: Literal[
        'AES256',
        'aws:kms',
        'aws:kms:dsse'
    ]
    SSECustomerAlgorithm: StrictStr
    SSECustomerKeyMD5: StrictStr
    SSEKMSKeyId: StrictStr
    SSEKMSEncryptionContext: StrictStr
    BucketKeyEnabled: StrictBool
    RequestCharged: Optional[
        Literal['requester']
    ]=None